<?php

namespace USKit\Base\Parts;

use USKit\Base\Interfaces\ViewModel;
use Infomaker\Everyware\NewsML\Objects\ImageItem;
use Infomaker\Everyware\Support\NewRelicLog;
use Infomaker\Imengine\CropData\AutoCropData;
use Infomaker\Imengine\CropData\CropResizeData;
use Infomaker\Imengine\Helpers\ImengineHelper;
use Infomaker\Imengine\Imengine;
use RuntimeException;

class Image implements ViewModel
{

    /**
     * @var ImageItem
     */
    private $image;

    public function __construct(ImageItem $image)
    {
        $this->image = $image;
    }

    public function getRatioUrl($width, $ratio = '16:9'): string
    {
        try {

            if ($this->image->hasCropForRatio($ratio)) {
                $crop = $this->image->getCropForRatio($ratio);

                $crop_handler = new CropResizeData($crop['width'], $crop['height']);

                // ERD 2022-08-23: Limit crop size to 2000px in either dimension.
                // Imengine fails with very large images and will return
                //   improperly sized crops.
                $crop_height = ImengineHelper::calculateRatioHeight($width, $ratio);

                if ($crop_height > 2000) {
                    [$ratio_width, $ratio_height] = explode(':', $ratio);
                    $width = (int)round((2000 / (int)$ratio_height) * (int)$ratio_width);
                }

                if ($width > 2000) {
                    $width = 2000;
                }

                $crop_handler
                    ->setCropX($crop['x'])
                    ->setCropY($crop['y'])
                    ->setWidth($width)
                    ->setHeight(ImengineHelper::calculateRatioHeight($width, $ratio));

                return Imengine::cropResize($crop_handler)->fromUuid($this->image->uuid);
            }

            if ($this->image->hasOriginalSizes()) {
                $crop_handler = new AutoCropData($this->image->width, $this->image->height, $width);

                $crop_handler->setRatio($ratio);

                return Imengine::autoCrop($crop_handler)->fromUuid($this->image->uuid);
            }

            return Imengine::ratio($width, $ratio)->fromUuid($this->image->uuid);

        } catch (RuntimeException $e) {
            NewRelicLog::error('Failed to fetch imengine url', $e);
        }

        return '';
    }

    private function createSrcset(array $sizes): string
    {
        $srcset = [];
        foreach ($sizes as $size) {
            if (is_int($size)) {
                $srcset[] = $this->getRatioUrl($size) . " {$size}w";
            }
        }

        return implode(',', $srcset);
    }

    /**
     * Generate URLs for all crops available to this image item - ERD 20220607
     * @param   array   $crops  All crop data available to this image item
     * @return  array           Key is crop name, value is imengine URL generatedx from the crop data
     */
    private function createCropSet(array $crops): array
    {
        $cropSet = [];

        foreach ($crops as $name => $data) {
            $width = $this->image->width;
            $cropSet[$name] = $this->getRatioUrl($width, $name);
        }

        return $cropSet;
    }

    public function getViewData(): array
    {
        return array_replace($this->image->toArray(), [
            'authors' => $this->image->get('authors'),
            'imengine_path' => get_theme_mod('image_endpoint_url') ?? null,
            'uuid' => $this->image->uuid,
            'width' => $this->image->width,
            'height' => $this->image->height,
            'src' => $this->getRatioUrl(1200),
            'src16x9' => $this->getRatioUrl(1200, '16:9'),
            'src3x2' => $this->getRatioUrl(1200, '3:2'),
            'src1x1' => $this->getRatioUrl(1200, '1:1'),
            'srcset' => $this->createSrcset([300, 600, 900, 1200, 1500]),
            'alignment' => $this->image->objectProperties['alignment'] ?? null,
            'size' => $this->image->objectProperties['size'] ?? null,
            'cropset' => $this->createCropSet($this->image->crop ?? []),
            'alttext'  => $this->image->alttext
        ]);
    }
}
